﻿using System;
using System.Collections.Generic;

namespace Nimm
{
    class PlayerComputer : PlayerBase
    {
        MoveGenerator moveGenerator;
        int maxSearchDepth = 19;
        int winning;

        public PlayerComputer(Board board, int winning)
            : base(board)
        {
            moveGenerator = new MoveGenerator(board);
            player = "Computer";
            this.winning = winning;
        }

        override public Move SearchMove(bool playerMaxHasToMove)
        {
            evalLeaf = Max(playerMaxHasToMove, maxSearchDepth);

            return bestMove;
        }

        int Max(bool playerMaxHasToMove, int searchDepth) 
        {
            if (searchDepth == 0 || NoPossibleMove())
                return Evaluate(playerMaxHasToMove);
            int bestScore = int.MinValue / 2; //-unendlich;
            List<Move> moveList = moveGenerator.GetMoves();
            foreach (var move in moveList)
            {
               board.Do(move);
               int eval = Min(!playerMaxHasToMove, searchDepth - 1);
               board.Undo(move);
               if (eval > bestScore)
               {
                   bestScore = eval;
                   if (searchDepth == maxSearchDepth)
                     bestMove = move;
               }
            }
            return bestScore;
        }

        int Min(bool playerMaxHasToMove, int searchDepth)
        {
            if (searchDepth == 0 || NoPossibleMove())
                return Evaluate(playerMaxHasToMove);
            int bestScore = int.MaxValue / 2;//unendlich;
            List<Move> moveList = moveGenerator.GetMoves();
            foreach (var move in moveList)
            {
                board.Do(move);
                int eval = Max(!playerMaxHasToMove, searchDepth - 1);
                board.Undo(move);
                if (eval < bestScore) 
                {
                      bestScore = eval;
                }
            }
            return bestScore;
        }

        private int Evaluate(bool playerMaxHasToMove)
        {
            int eval = 0;
            if (board.NumberOfMatches == 0)
                eval = winning;
            return playerMaxHasToMove ? -eval : eval;
        }

        private bool NoPossibleMove()
        {
            return board.NumberOfMatches == 0;
        }
    }
}
